package control;

import model.CollisionData;
import model.CollisionSummary;
import model.ManModel;
import model.Model;
import view.Obstacle;
import view.View;

import java.util.*;

public class Controller {

    private static Random loto = new Random(Calendar.getInstance().getTimeInMillis());
    protected Model model;
    protected View view;
    protected ControllerKeyboard keyboard;
    protected ControllerAnimation animator;
    private List<Obstacle> filteredObstacles;

    public Controller(Model model, View view) {
        this.model = model;
        this.view = view;
        keyboard = new ControllerKeyboard(model,view,this);
        animator = new ControllerAnimation(model, view, this);
        filteredObstacles = new ArrayList<>();

    }

    public void startAnimation() {
        animator.start();
    }
    public void stopAnimation() {
        animator.stop();
    }

    public void step() {

        double srcX = model.getMan().getX();
        double srcY = model.getMan().getY();
        //System.out.println(model.getFrameCounter()+" - initial state = "+model.getMan().getState()+", pos. in "+srcX+","+srcY);

        // If abnormal behavior is observed, uncomment next line to check abnormal collision case
        checkAbnormalCollision();

        if ((model.getMan().getState() == ManModel.STATE_IDLE) || (model.getMan().getState() == ManModel.STATE_MOVING) || (model.getMan().getState() == ManModel.STATE_SLIDING)) {
            if (!model.isKeyPressed()) {
                // set idle only if moving at the end of previous frame and no key is pressed
                // if sliding, then continue sliding.
                if (model.getMan().getState() == ManModel.STATE_MOVING) {
                    model.getMan().setStateIdle();
                }
            }
            else {
                int idxLeft = model.indexKeyPressed("Left");
                int idxRight = model.indexKeyPressed("Right");
                int idxSpace = model.indexKeyPressed("Space");
                // left is newly pressed compared to right
                if (idxLeft > idxRight) {
                    if (idxSpace != -1) {
                        model.getMan().setStateJump(Model.LEFT);
                    }
                    else {
                        model.getMan().setStateMove(Model.LEFT);
                    }
                }
                // right is newly pressed compared to left
                else if (idxLeft < idxRight) {
                    if (idxSpace != -1) {
                        model.getMan().setStateJump(Model.RIGHT);
                    }
                    else {
                        model.getMan().setStateMove(Model.RIGHT);
                    }
                }
                // none is pressed, thus they are both equals to -1
                else {
                    if (idxSpace != -1) {
                        model.getMan().setStateJump(Model.UP);
                    }
                }
            }
        }
        /* first step: try to filter obstacles to reduce the number of obstacles that may interact with the Man
           according to the current state, the "window"
         */
        // compute the next position a first time,
        model.getMan().computeNextPosition();
        //System.out.println(model.getFrameCounter()+" - step 1 state = "+model.getMan().getState()+", should go to "+destX+","+destY);

        filterObstacles();
        double dx = model.getMan().getDx();
        double dy = model.getMan().getDy();
        if ((model.getMan().getState() == ManModel.STATE_SLIDING) && ((dx == 0.0) || (dy == 0.0))) {
            model.getMan().setStateIdle();
        }
        /*
        System.out.print(model.getFrameCounter()+" - remaining obstacles: ");
        for (Obstacle o : filteredObstacles) {
            System.out.print(o+" | ");
        }
        System.out.println();
        */

        if (model.getMan().isCheckFloor()) {
            checkFloor();
        }

        //System.out.println(model.getFrameCounter()+" - step 2 state = "+model.getMan().getState());

        /* at this point, if the Man is idel, or at the first step of sliding, do nothing.
           Otherwise, check for collisions
         */
        boolean doCollisions = true;
        if ((model.getMan().getState() == ManModel.STATE_IDLE) ||
                ((model.getMan().getState() == ManModel.STATE_SLIDING) && (model.getMan().isIdleToSlide())) )   {
            doCollisions = false;
        }

        if (doCollisions) {
            // now state is fixed for the current frame, we can compute the real next position
            model.getMan().computeNextPosition();
            /*
            double destX = (srcX + model.getMan().getDx());
            double destY = (srcY + model.getMan().getDy());
            System.out.println(model.getFrameCounter()+" - before collision test, state = "+model.getMan().getState()+", should go to "+destX+","+destY);
             */
            checkCollisions();
        }
        model.update();
        view.update();
        model.resetFlags();
    }

    public void filterObstacles() {
        filteredObstacles.clear();
        view.getManLook().updateFilterArea();
        for (Obstacle o : view.getObstacles()) {
            if (view.getManLook().mayCollideWithObstacle(o)) {
                filteredObstacles.add(o);
            }
        }
    }

    /* NB: this method is normally used only for debug
     */
    public void checkAbnormalCollision() {

        // check abnormal case: man is already in collision in its current position
        boolean collide = false;
        for (Obstacle o : view.getObstacles()) {
            CollisionData data = view.getManLook().checkCollision(o);
            if ( data.width != -1) {
                collide = true;
                double srcX = model.getMan().getX();
                double srcY = model.getMan().getY();
                System.out.println(model.getFrameCounter()+" - ABNORMAL COLLISION with "+o+" in state = "+model.getMan().getState()+", pos. in "+srcX+","+srcY);
                System.exit(1);
                break;
            }
        }
    }

    public void checkFloor() {

        // checking a floor is done only when idle.
        if (model.getMan().getState() != ManModel.STATE_IDLE) return;
        // get current position
        int srcX = (int) model.getMan().getX();
        int srcY = (int) model.getMan().getY();


        view.getManLook().setHitBoxPosition(srcX, srcY+1);
        CollisionSummary summary = computeCollisions(filteredObstacles);
        int quadrants = summary.getQuadrants();
        // no collision => no floor
        if (!summary.isCollision()) {
            // lets fall
            System.out.println(model.getFrameCounter()+" - no floor => falling");
            model.getMan().setStateFall();
        }
        // only bottom-left of Man is on the floor => lets move a bit the man to the right to slide
        // search for the position at right where there is more collision or where the Man is blocked
        else if (quadrants == CollisionData.QUADRANT_BOTTOMLEFT) {

            int stepX = searchHorizontalPosition(srcX, srcY+1, Model.RIGHT);
            if (stepX >0) {
                System.out.println(model.getFrameCounter() + " - just bottom left => should slide right, by moving of "+stepX);
                model.getMan().setStateSlide(srcX+stepX, srcY+1, true);
            }
        }
        // only bottom-right of Man is on the floor => lets move a bit the man to the left to slide
        // search for the position at left where there is more collision or where the Man is blocked
        else if (quadrants == CollisionData.QUADRANT_BOTTOMRIGHT) {
            int stepX = searchHorizontalPosition(srcX, srcY+1, Model.LEFT);
            if (stepX > 0) {
                System.out.println(model.getFrameCounter() + " - just bottom right => should slide left by moving of "+ stepX);
                model.getMan().setStateSlide(srcX-stepX, srcY+1, true);
            }
        }
        // reset hitbox to its original position so that further collision detection are not abnormal
        view.getManLook().setHitBoxPosition(srcX, srcY);
        //System.out.println(model.getFrameCounter()+" - on the floor => still");
    }

    /* starting from srcX, srcY, search for a position where there is no collision at all, or
       the Man is blocked by an obstacle in direction of search.
       returns :
         the number of steps to reach a good position or minus the number of steps to be blocked.
     */
    private int searchHorizontalPosition(double srcX, double srcY, int direction) {
        boolean stop = false;
        boolean blocked = false;
        int stepX = 0;
        while (!stop) {
            view.getManLook().setHitBoxPosition(srcX, srcY);
            CollisionSummary summary = computeCollisions(filteredObstacles);
            if (!summary.isCollision()) {
                stop = true;
            }
            // if we try to find a position at right, then if there is a collision in right quadrants, then it is not possible to go more at the right.
            else if ( (direction == Model.RIGHT) &&  (summary.isCollisionRight()) ) {
                blocked = true;
                stop = true;
            }
            // if we try to find a position at left, then if there is a collision in left quadrants, then it is not possible to go more at the left.
            else if ( (direction == Model.LEFT) &&  (summary.isCollisionLeft()) ) {
                blocked = true;
                stop = true;
            }
            else {
                if (direction == Model.RIGHT) {
                    srcX+=1.0;
                }
                else {
                    srcX-=1.0;
                }
                stepX++;
            }
        }
        if (blocked) {
            return -1;
        }
        return stepX;
    }

    /* starting from srcX, srcY, search for a position where there is no collision at all, or
       the Man is blocked by an obstacle in direction of search.
       returns :
         the number of steps to reach a good position or -1 if blocked without finding a good position
     */
    private int searchVerticalPosition(double srcX, double srcY, int direction) {
        boolean stop = false;
        boolean blocked = false;
        int stepY = 0;
        while (!stop) {
            view.getManLook().setHitBoxPosition(srcX, srcY);
            CollisionSummary summary = computeCollisions(filteredObstacles);
            if (!summary.isCollision()) {
                stop = true;
            }
            // if we try to find a position at bottom, then if there is a collision in bottom quadrants, then it is not possible to go more down.
            else if ( (direction == Model.DOWN) &&  (summary.isCollisionBottom()) ) {
                blocked = true;
                stop = true;
            }
            // if we try to find a position up, then if there is a collision in top quadrants, then it is not possible to go upper.
            else if ( (direction == Model.UP) &&  (summary.isCollisionTop()) ) {
                blocked = true;
                stop = true;
            }
            else {
                if (direction == Model.DOWN) {
                    srcY+=1.0;
                }
                else {
                    srcY-=1.0;
                }
                stepY++;
            }
        }
        if (blocked) {
            return -1;
        }
        return stepY;
    }

    /* starting from srcX, srcY, search the first position for which there is a collision
       returns :
         the number of steps to reach the collision or -1 if limitY is reached
     */
    private int searchVerticalCollision(double srcX, double srcY, double limitY, int direction) {
        boolean stop = false;
        boolean limit = false;
        int stepY = 0;
        while (!stop) {
            view.getManLook().setHitBoxPosition(srcX, srcY);
            CollisionSummary summary = computeCollisions(filteredObstacles);
            if (summary.isCollision()) {
                stop = true;
            }
            else {
                if (direction == Model.DOWN) {
                    srcY+=1.0;
                    if (srcY>limitY) {
                        stop = true;
                        limit = true;
                    }
                }
                else {
                    srcY-=1.0;
                    if (srcY<limitY) {
                        stop = true;
                        limit = true;
                    }
                }
                stepY++;
            }
        }
        if (limit) {
            return -1;
        }
        return stepY;
    }

    public void checkCollisions() {

        if (model.getMan().getState() == ManModel.STATE_MOVING) {
            checkCollisionsForMove();
        }
        else if (model.getMan().getState() == ManModel.STATE_SLIDING) {
            checkCollisionsForSlide();
        }
        else {
            checkCollisionsForFallOrJump();
        }
    }

    private void checkCollisionsForSlide() {
        // get current & next position
        double srcX = model.getMan().getX();
        double srcY = model.getMan().getY();
        double dx = model.getMan().getDx();
        double dy = model.getMan().getDy();
        double maxSpeed = Math.sqrt(dx*dx+dy*dy);
        double dyFall = model.getMan().getDyFall();
        double destX = srcX+dx;
        double destY = srcY+dy;
        int gapX = 0;
        int incX = 1;
        if (model.getMan().getDirection() == Model.LEFT) {
            incX = -1;
        }
        int gapY = 0;
        double absdx = dx;
        if (dx < 0) absdx = -dx;
        List<int[]> pixelPath = bresenham((int)srcX, (int)srcY, (int)destX, (int)destY);

        boolean blocked = false;
        boolean tooFast = false;
        int p;
        for (p = 1;p<pixelPath.size();p++) {
            int[] t = pixelPath.get(p);
            // compute acquired speed in the current position
            double curSpeed = Math.sqrt((t[0]+gapX - srcX)*(t[0]+gapX - srcX) + (t[1]+gapY - srcY)*(t[1]+gapY - srcY));
            if (curSpeed > maxSpeed) {
                tooFast = true;
                break;
            }
            view.getManLook().setHitBoxPosition(t[0]+gapX, t[1]+gapY);
            CollisionSummary summary = computeCollisions(filteredObstacles);

            // if there is a collision in the whole bottom, it means the slope becomes far less important => search upward the final position
            // if this is in one of top quadrants, the sprite can neither go further
            if ( (summary.isCollisionWholeBottom()) || (summary.isCollisionTop())) {
                System.out.println("slide - collide on top or whole bottom");
                blocked = true;
                break;
            }
            // direction is left and collision in bottom left => sprite cannot climb when sliding thus blocked
            else if ((model.getMan().getDirection() == Model.LEFT) && (summary.isCollisionBottomLeft()) ) {
                System.out.println("slide - going left and collide bottom left");
                blocked = true;
                break;
            }
            // direction is right and collision in bottom right => sprite cannot climb when sliding thus blocked
            else if ((model.getMan().getDirection() == Model.RIGHT) && (summary.isCollisionBottomRight())) {
                System.out.println("slide - going right and collide bottom right");
                blocked = true;
                break;
            }
            // in other collision cases
            else if (summary.isCollision()) {
                // if slope > 45°, search good position left/right
                if (dy > absdx) {
                    int stepX = searchHorizontalPosition(t[0]+gapX+incX, t[1]+gapY, model.getMan().getDirection());
                    if (stepX < 0) {
                        System.out.println("slide - big slope, blocked while searching for good x position");
                        blocked = true;
                        break;
                    }
                    if (incX == 1) {
                        gapX += (stepX+1);
                    }
                    else {
                        gapX -= (stepX+1);
                    }
                }
                // if slope < 45°, search good position up
                else {
                    int stepY = searchVerticalPosition(t[0]+gapX, t[1]+gapY-1, Model.UP);
                    if (stepY < 0) {
                        System.out.println("slide - low slope, blocked while searching for good y position");
                        blocked = true;
                        break;
                    }
                    gapY -= (stepY+1);
                }
            }
        }

        if (blocked) {
            int[] pos = pixelPath.get(p-1);
            System.out.println(model.getFrameCounter()+" - blocked while sliding, must put in "+(pos[0]+gapX)+","+(pos[1]+gapY));
            // set hitbox to its final dest, even if it is the same as at the beginning of the current frame.
            // Indeed, it may have move for testing collision. Since the view is not refreshed if the Man does not move,
            // the hitbox is not reset to its original position. This is why it must be done here, in case of no move.
            view.getManLook().setHitBoxPosition(pos[0]+gapX,pos[1]+gapY);
            model.getMan().setStateBlocking(pos[0]+gapX,pos[1]+gapY);
            return;
        }
        if (tooFast) {
            /* in case of sliding that starts with a very low speed, the maximal
               speed is reached as soon as the sprite moves from 1 pixel. In this case,
               its position is the initial one, srcX,srcY, tho keep its exact coordinates
               instead of those converted to an int, as given by p[0],p[1]
             */
            if (p == 1) {
                destX = srcX;
                destY = srcY;
            }
            else {
                int[] pos = pixelPath.get(p - 1);
                destX = pos[0];
                destY = pos[1];
            }
            System.out.println(model.getFrameCounter()+" - max speed reached at step "+p+" out of "+pixelPath.size()+ " while sliding, so stop loop at "+(destX+gapX)+","+(destY+gapY));
        }

        /* if the Man is "in the air", must search for a collision down, until srcY+dyFall. But if there is a collision
         in srcY+dyFall+1 it is also ok because the man is in contact with an obstacle in srcY+dyFall. So we have to search
         until srcY+dyFall+1 and if there are no collision, it falls.
         */
        int stepY = searchVerticalCollision(destX+gapX, destY+gapY+1, srcY+dyFall+1, Model.DOWN);
        if (stepY < 0) {
            // considered to be falling, because no collision detected at src+dyFall
            System.out.println(model.getFrameCounter()+" - fall after slide, limit in "+(srcY+dyFall)+" reached, so must put in "+(destX+gapX)+","+(srcY+dyFall));
            view.getManLook().setHitBoxPosition(destX+gapX, srcY+dyFall);
            model.getMan().setStateFallAfterMoveOrSlide(destX+gapX, srcY+dyFall);
        }
        else {
            // update dest with gaps & steps
            destX = destX+gapX;
            if (destY+gapY+stepY < srcY) {
                destY = srcY;
            }
            else {
                destY = destY+gapY+stepY;
            }
            System.out.println(model.getFrameCounter()+" - keep sliding, final step = "+stepY+" so must put in "+destX+","+destY);
            view.getManLook().setHitBoxPosition(destX,destY);
            model.getMan().setStateSlide(destX,destY, false);
        }
    }

    private void checkCollisionsForFallOrJump() {

        double srcX = model.getMan().getX();
        double srcY = model.getMan().getY();
        double dx = model.getMan().getDx();
        double dy = model.getMan().getDy();
        double destX = srcX+dx;
        double destY = srcY+dy;
        List<int[]> pixelPath = bresenham((int)srcX, (int)srcY, (int)destX, (int)destY);
        int i=0;
        boolean blocked = false;
        boolean fall = false;
        for (i = 1;i<pixelPath.size();i++) {
            int[] t = pixelPath.get(i);
            view.getManLook().setHitBoxPosition(t[0], t[1]);
            CollisionSummary summary = computeCollisions(filteredObstacles);
            if (summary.isCollisionTop()) {
                System.out.println("fall/jump - collide on top");
                fall = true;
                break;
            }
            else if (summary.isCollisionWholeBottom()) {
                System.out.println("fall/jump - collide on whole bottom");
                blocked = true;
                break;
            }
            else if (summary.isCollisionLeft()) {
                // sprite is strictly going left
                if (dx < 0) {
                    System.out.println("fall/jump - going left and collide on left => blocked");
                    blocked = true;
                }
                // sprite is going down or right
                else {
                    int[] p = pixelPath.get(i - 1);
                    System.out.println(model.getFrameCounter() + " - sliding right after fall/jump, must put in " + p[0] + "," + p[1]);
                    view.getManLook().setHitBoxPosition(p[0], p[1]);
                    // because of conversion to int, p[0] may be < srcX, which is not posible
                    // in this case, we set the Man at srcX, instead of p[0]
                    if (p[0] < srcX) {
                        model.getMan().setStateSlideAfterFallOrJump(srcX, p[1], Model.RIGHT);

                    }
                    else {
                        model.getMan().setStateSlideAfterFallOrJump(p[0], p[1], Model.RIGHT);
                    }
                    return;

                }
                break;
            }
            else if (summary.isCollisionRight()) {
                // sprite is strictly going right
                if (dx > 0) {
                    System.out.println("fall/jump - going right and collide on right  => blocked");
                    blocked = true;
                }
                // sprite is going down or left
                else {
                    int[] p = pixelPath.get(i-1);
                    System.out.println(model.getFrameCounter()+" - sliding left after fall/jump, must put in "+p[0]+","+p[1]);
                    view.getManLook().setHitBoxPosition(p[0], p[1]);
                    model.getMan().setStateSlideAfterFallOrJump(p[0], p[1], Model.LEFT);
                    return;
                }
                break;
            }
        }
        if (blocked) {
            int[] p = pixelPath.get(i-1);
            System.out.println(model.getFrameCounter()+" - collide in state "+model.getMan().getState()+" must put idle in "+p[0]+","+p[1]);
            view.getManLook().setHitBoxPosition(p[0], p[1]);
            model.getMan().setStateBlocking(p[0], p[1]);
            return;
        }
        else if (fall) {
            int[] p = pixelPath.get(i-1);
            System.out.println(model.getFrameCounter()+" - collide in state "+model.getMan().getState()+" must put falling in "+p[0]+","+p[1]);
            view.getManLook().setHitBoxPosition(p[0], p[1]);
            model.getMan().setStateFall(p[0], p[1]);
            return;
        }
    }

    private void checkCollisionsForMove() {


        /* see README.txt in /doc directory for an explanation
         */
        // get current & next position
        double srcX = model.getMan().getX();
        double srcY = model.getMan().getY();
        double dx = model.getMan().getDx();
        double dy = model.getMan().getDy();
        double destY = srcY;
        int direction = 1;

        if (dx == 0) {
            return;
        }
        else if (dx < 0) {
            dx = -dx;
            direction = -1;
        }
        // normally, dist must be equals to currentStepX in ManModel

        int height = 0; // the increment in y added to srcY
        int p;
        boolean blocked = false;
        boolean falling = false;
        // start at 1 because we don't have to test the current position.
        for (p = 1;p<=dx;p++) {
            view.getManLook().setHitBoxPosition(srcX+direction*p, srcY+height);
            CollisionSummary summary = computeCollisions(filteredObstacles);
            // collision on bottom only
            if (summary.isCollisionBottom()) {
                if (summary.isCollisionTop()) {
                    System.out.println("moving - blocked at top & bottom");
                    blocked = true;
                    break;
                }
                // collision 1 pixel plus haut => blocked
                if (checkCollisionAt(srcX+direction*p, srcY+height-1, filteredObstacles)) {
                    System.out.println("moving - blocked at bottom but cannot go up");
                    blocked = true;
                    break;
                }
                else {
                    System.out.println("moving - climb one pixel");
                    height--;
                }
            }
            // no collision, or just at top
            else {
                // test collision one pixel below
                boolean collisionOneBelow = checkCollisionAt(srcX+direction*p, srcY+height+1, filteredObstacles);

                // collision on top & bottom: impossilbe to get down
                if (summary.isCollisionTop() && collisionOneBelow) {
                    System.out.println("moving - blocked at top but cannot go down");
                    blocked = true;
                    break;
                }
                else if (!collisionOneBelow) {
                    // test collision two pixels below
                    if(checkCollisionAt(srcX+direction*p, srcY+height+2, filteredObstacles)) {
                        System.out.println("moving - get down of one pixel");
                        height++;
                    }
                    else {
                        System.out.println("moving - do fall");
                        falling = true;
                        break;
                    }
                }

            }
        }
        if (blocked) {
            view.getManLook().setHitBoxPosition(srcX + direction * (p - 1), srcY + height);
            model.getMan().setStateBlocking(srcX + direction * (p - 1), srcY + height);
        }
        else if (falling) {
            view.getManLook().setHitBoxPosition(srcX + direction * p, srcY + height);
            model.getMan().setStateFallAfterMoveOrSlide(srcX + direction * p, srcY + height);
        }
        else {
            // do not change the state, only the position
            view.getManLook().setHitBoxPosition(srcX + direction * dx, srcY + height);
            model.getMan().setNewPosition(srcX+direction*dx, srcY+height);
        }
    }


    // CAUTION : this version of bresenham returns a list of points from x1,y1 to x2,y2
    private List<int[]> bresenham(int x1, int y1, int x2, int y2) {
        if (Math.abs(y2-y1) < Math.abs(x2-x1)) {
            if (x1 < x2) {
                return bresenhamLow(x1, y1, x2, y2, false);
            }
            else {
                return bresenhamLow(x2, y2, x1, y1, true);
            }
        }
        else {
            if (y1 < y2) {
                return bresenhamHigh(x1, y1, x2, y2, false);
            }
            else {
                return bresenhamHigh(x2, y2, x1, y1, true);
            }
        }
    }

    private List<int[]> bresenhamLow(int x1, int y1, int x2, int y2, boolean reversed) {
        List<int[]> lst = new ArrayList<>();
        int dx = x2 - x1;
        int dy = y2 - y1;
        int yi = 1;
        if (dy < 0) {
            yi = -1;
            dy = -dy;
        }

        int D = (2 * dy) - dx;
        int y = y1;

        for (int x = x1; x<=x2;x++) {
            int[] tab = new int[2];
            tab[0] = x;
            tab[1] = y;
            if (reversed) {
                lst.add(0,tab);
            }
            else {
                lst.add(tab);
            }
            if (D > 0 ) {
                y = y + yi;
                D = D + (2 * (dy - dx));
            }
            else {
                D = D + 2*dy;
            }
        }
        return lst;
    }
    private List<int[]> bresenhamHigh(int x1, int y1, int x2, int y2, boolean reversed) {
        List<int[]> lst = new ArrayList<>();
        int dx = x2 - x1;
        int dy = y2 - y1;
        int xi = 1;
        if (dx < 0) {
            xi = -1;
            dx = -dx;
        }

        int D = (2 * dx) - dy;
        int x = x1;

        for (int y = y1; y<=y2;y++) {
            int[] tab = new int[2];
            tab[0] = x;
            tab[1] = y;
            if (reversed) {
                lst.add(0,tab);
            }
            else {
                lst.add(tab);
            }
            if (D > 0 ) {
                x = x + xi;
                D = D + (2 * (dx - dy));
            }
            else {
                D = D + 2*dx;
            }
        }
        return lst;
    }

    private CollisionSummary computeCollisions(List<Obstacle> obstacles) {
        CollisionSummary summary = new CollisionSummary();
        int quadrants = 0;
        for (Obstacle o : obstacles) {
            CollisionData data = view.getManLook().checkCollision(o);
            // only add to summary collisions
            if (data.width >= 0) {
                quadrants = view.getManLook().getCollisionQuadrants(data);
                summary.addCollision(data, quadrants);
            }
        }
        return summary;
    }

    // simple test, when detailed info are not needed
    private boolean checkCollisionAt(double x, double y, List<Obstacle> obstacles) {
        view.getManLook().setHitBoxPosition(x,y);
        boolean collide = false;
        for (Obstacle o : obstacles) {
            CollisionData data = view.getManLook().checkCollision(o);
            if (data.width != -1) {
                collide = true;
                break;
            }
        }
        return collide;
    }
}
