package model;

public class ManModel {

    public final static int STATE_IDLE = 0;
    public final static int STATE_MOVING = 1;
    public final static int STATE_JUMPING = 2; // jump, but not simply falling
    public final static int STATE_FALLING = 3; // just fall  from a platform
    public final static int STATE_SLIDING = 4; // when obstacle slope leads to slide
    public final static int STATE_BLOCKING = 5; // hit something after a fall/jump/move

    public final static double gravity = 2; // used to manage speedY when jumping/falling
    public final static double initialSpeedYJump = -23; // used init. speedY when jumping
    public final static double friction = 0.5; // friction coef applied to decrease speed at each frame when sliding

    // baseStepX represents the default number of pixels the Man is displaced when moving left/right.
    private final double baseStepX = 6;
    /* currentStepX represents the current number of pixels the Man is displaced when moving left/right,
       taking into account bonus/malus applied to baseStepX. If there are no modifiers, then its value
       is baseStepX.
       IMPORTANT: when the Man is just moving (not sliding/falling/jumping), it does in fact small steps and is
       not considered to move regularly and to have a speed. thus, currentStepX is used to update the Man's position when moving.
       When sliding/falling/jumping, it has a speed and speedX/speedY are used to update its position.
     */
    private double currentStepX;

    private double speedX;
    private double speedY;
    // position in absolute space (NB: scene space = absolute space for convenience)
    private double x;
    private double y;
    // the displacement in X/Y during the frame, taking into account the curret state
    private double dx;
    private double dy;
    private double dyFall; // used during sliding to get the maximum dy in case of the current displacement becomes a fall
    // the new position when a collision occurred or when just moving: computed by the controller
    private double newPositionX;
    private double newPositionY;
    // state
    private int state;
    private int direction; // the direction of the current displacement
    // flags for different events
    private boolean positionChanged;
    private boolean collisionOccurred;
    private boolean endCollisionOccurred;
    private boolean idleToSlide; // set to true only if the Man was idle and it starts to slide
    private boolean checkFloor; // set to true only when st to idle after move/fall/jump/slide stops, and thus leads to idle
    // counter associated to flags
    private int counterCollision;

    public ManModel() {
        currentStepX = baseStepX;
        speedX = 0.0;
        speedY = 0.0;
        x = 0;
        y = 0;
        dx = 0;
        dy = 0;
        dyFall = 0;
        newPositionX = 0.0;
        newPositionY = 0.0;
        positionChanged = false;
        collisionOccurred = false;
        endCollisionOccurred = false;
        counterCollision = 0;
        idleToSlide = false;
        checkFloor = true;
        state = STATE_IDLE;
    }

    public double getX() {
        return x;
    }

    public double getY() {
        return y;
    }

    public double getDx() {
        return dx;
    }

    public double getDy() {
        return dy;
    }

    public double getDyFall() {
        return dyFall;
    }

    public double getCurrentStepX() {
        return currentStepX;
    }

    public int getDirection() {
        return direction;
    }

    public boolean isPositionChanged() {
        return positionChanged;
    }

    public boolean isIdleToSlide() {
        return idleToSlide;
    }

    public void resetFlags() {
        positionChanged = false;
        collisionOccurred = false;
        endCollisionOccurred = false;
    }

    public int getState() {
        return state;
    }

    public void setNewPosition(double newPositionX, double newPositionY) {
        this.newPositionX = newPositionX;
        this.newPositionY = newPositionY;
    }

    public void setStateIdle() {
        checkFloor = true;
        stop();
    }

    public boolean isCheckFloor() {
        return checkFloor;
    }

    public void setStateBlocking(double collisionX, double collisionY) {
        state = STATE_BLOCKING;
        setNewPosition(collisionX, collisionY);
        collisionOccurred = true;
        counterCollision = 20;
    }

    public boolean isCollisionOccurred() {
        return collisionOccurred;
    }

    public boolean isEndCollisionOccurred() {
        return endCollisionOccurred;
    }

    public void put(double x, double y) {
        if ((this.x != x) || (this.y != y)) {
            positionChanged = true;
            this.x = x;
            this.y = y;
        }
    }

    public void setStateMove(int direction) {
        state = STATE_MOVING;
        this.direction = direction;
    }

    public void setStateJump(int direction) {
        // NB : can only do a jump when not jumping or falling
        //System.out.println("do jump");
        state = STATE_JUMPING;
        this.direction = direction;
        if (direction == Model.RIGHT) {
            speedX = currentStepX * 1.5;
        }
        else if (direction == Model.LEFT) {
            speedX = -currentStepX * 1.5;
        }
        else {
            this.direction = Model.UP;
            speedX = 0.0;
        }
        speedY = initialSpeedYJump;
    }

    private void jump() {
        if (state == STATE_JUMPING) {
            x += speedX;
            speedY += gravity;
            y += speedY;
            positionChanged = true;
        }
    }

    public void setStateFall() {
        state = STATE_FALLING;
        direction = Model.DOWN;
        speedX = 0.0;
        speedY = 0.0;
    }

    public void setStateFall(double newPositionX, double newPositionY) {
        state = STATE_FALLING;
        x = newPositionX;
        y = newPositionY;
        direction = Model.DOWN;
        speedX = 0.0;
        speedY = -gravity;
    }
    // after sliding/moving update x & y speed from the new position where the Man must be put
    public void setStateFallAfterMoveOrSlide(double newPositionX, double newPositionY)  {
        state = STATE_FALLING;
        speedX = newPositionX - x;
        // since gravity will also be added to get the new position, subtract it from speedY
        speedY = newPositionY - y - gravity;
        if (speedX < 0) {
            direction = Model.LEFT;
        }
        else if (speedX > 0) {
            direction = Model.RIGHT;
        }
        else {
            direction = Model.DOWN;
        }

    }

    private void fall() {
        if (state == STATE_FALLING) {
            x += speedX; // in cas of the is really a fall with an X speed (e.g. after sliding)
            speedY += gravity;
            y += speedY;
            positionChanged = true;
        }
    }

    public void setStateSlide(double newPositionX, double newPositionY, boolean idleToSlide) {
        this.idleToSlide = idleToSlide;
        state = STATE_SLIDING;
        this.newPositionX = newPositionX;
        this.newPositionY = newPositionY;
        speedX = newPositionX - x;
        speedY = newPositionY - y;
        if (speedX >= 0) {
            direction = Model.RIGHT;
        }
        else {
            direction = Model.LEFT;
        }
    }

    public void setStateSlideAfterFallOrJump(double newPositionX, double newPositionY, int direction) {
        this.newPositionX = newPositionX;
        this.newPositionY = newPositionY;
        this.direction = direction;
        // if no speed at all (because sliding occurs during the start of a fall) init. speed
        if (speedX == 0.0) {
            speedX = newPositionX - x;
        }
        if (speedY == 0.0) {
            speedY = newPositionY - y;
        }
        state = STATE_SLIDING;
    }

    // get the current position, i.e. at the beginning of the frame
    public double[] getPosition() {
        double[] tab = {getX(), getY()};
        return tab;
    }

    // get the next position, taking into account the state
    public void computeNextPosition() {
        if (state == STATE_MOVING) {
            if (direction == Model.LEFT) {
                dx = -currentStepX;
                dy = 0;
            }
            else {
                dx = currentStepX;
                dy = 0;
            }
        }
        else if (state == STATE_SLIDING) {
            double theta = Math.atan2(speedY, speedX);
            // acceleration depends on the slope
            double Acc = gravity*Math.sin(theta);
            //System.out.println(Acc-friction);
            dx = speedX + (Acc-friction)*Math.cos(theta)/2.0;
            // dx cannot go over/under 0 if it is not of the same sign as speedX
            if ( ((dx>=0) && (speedX <0)) || ((dx<=0) && (speedX >0))) {
                dx = 0.0;
            }
            dy = speedY + (Acc-friction)*Math.sin(theta)/2.0;
            dyFall = speedY + gravity;
            //System.out.println("sliding - slope = "+Math.toDegrees(theta)+" , speedx = "+speedX+", speedY = "+speedY+" dx = "+dx+" , dy = "+dy);
        }
        else if ((state == STATE_FALLING)  || (state == STATE_JUMPING)) {
            dx = speedX;
            dy = speedY+gravity;
        }
        else {
            dx = 0;
            dy = 0;
        }
    }

    public void increaseSpeedY(double value) {
        speedY += value;
    }

    public void stop() {

        speedX = 0.0;
        speedY = 0.0;
        state = STATE_IDLE;
    }

    public void update() {

        counterCollision--;
        if ( counterCollision == 0) {
            endCollisionOccurred = true;
        }

        if (state == STATE_BLOCKING) {
            // controller already computed the new position
            // thus just signal that to the view
            setStateIdle();
            put(newPositionX, newPositionY);
        }
        else if  (state == STATE_MOVING) {
            put(newPositionX, newPositionY);
        }
        else if (state == STATE_SLIDING) {
            if (idleToSlide) {
                idleToSlide = false;
            }
            put(newPositionX, newPositionY);
        }
        else if (state == STATE_FALLING) {
            fall();
        }
        else if (state == STATE_JUMPING) {
            jump();
        }
        else {
            // still idle at the ned of the current frame => no need th recheck the floor
            checkFloor = false;
            stop();
        }
    }
}
