package view;

import javafx.geometry.Bounds;
import javafx.scene.Group;
import javafx.scene.paint.Color;
import javafx.scene.shape.Circle;
import javafx.scene.shape.Rectangle;
import javafx.scene.shape.Shape;
import model.CollisionData;
import model.CollisionSummary;
import model.ManModel;

public class ManLook {
    private Group body;
    // shapes that constitute the body
    private Circle outer;
    private Circle inner;
    private double radius;

    private Group hitBox;
    // shape that constitute the hitBox
    private Circle hitCircle;

    // a transparent rectangle is used to select which obstacle may interact with the Man
    // its position & size will be set at the beginning of each frame (see Control.setp() )
    // according to the current state & position of the Man.
    private Rectangle filterArea;

    private ManModel manModel;

    // the global view (to gain access to width, height)
    private View view;

    public ManLook(ManModel manModel, double radius, View view) {

        this.radius = radius;
        this.manModel = manModel;
        this.view = view;
        // define the body
        body = new Group();
        outer = new Circle();
        outer.setRadius(radius);
        outer.setFill(Color.BLACK);
        outer.setCenterX(radius);
        outer.setCenterY(radius);
        inner = new Circle();
        inner.setRadius(5);
        inner.setFill(Color.WHITE);
        inner.setCenterX(radius);
        inner.setCenterY(radius);
        body.getChildren().addAll(outer, inner);
        // define the hitbox : in the present case, a circle that matches outer boundaries.
        hitBox = new Group();
        hitCircle = new Circle();
        hitCircle.setRadius(radius);
        // NB: in case of hitCircle is included in the scene, set it transparent
        hitCircle.setFill(Color.TRANSPARENT);
        hitCircle.setCenterX(radius);
        hitCircle.setCenterY(radius);
        hitBox.getChildren().addAll(hitCircle);
        // define the filterArea
        // NB: in case of filterArea is included in the scene, set it transparent
        filterArea = new Rectangle();
        filterArea.setFill(Color.TRANSPARENT);
    }

    public Group getBody() {
        return body;
    }

    public void setHitBoxPosition(double x, double y) {
        hitBox.setTranslateX(Math.floor(x));
        hitBox.setTranslateY(Math.floor(y));
    }

    public void updateFilterArea() {
        /* this method is called at the beginning of each frame, but after testing key pressed.
           Thus, the sprite may be in all possible states (except blocking) before executing this method

           1 - if the sprite is idle, it may pass to falling, or sliding state.
               - if sliding, the Man will go at most 2 pixels down, but the number in width is unknown and depends on the slope that
               makes the Man slide. Nevertheless, since this slope must be sufficient to obtain a slide, we can bet that the Man won't move
               more than its own width
               - if falling, the Man will go at most 2 pixel down

            2 - if the Man is moving, it may go left/right no more than baseStepX pixels, and the same in height since it cannot climb/fall of more than 45°.
            3 - If the Man is sliding, the width of the filter area is unknown but since speedY may be high, we just use the entiere scene width.
                For the height, it is determined by the next position in Y.
             4 - if the Man is falling/jumping, the filter area size&pos. can be determined from the current & next position
             */

        double srcX = manModel.getX();
        double srcY = manModel.getY();
        double destX = srcX + manModel.getDx();
        double destY = srcY + manModel.getDy();


        if (manModel.getState() == ManModel.STATE_IDLE) {
            filterArea.setX(srcX-2.0*radius);
            filterArea.setY(srcY);
            filterArea.setWidth(6.0*radius); // width
            filterArea.setHeight(2.0*radius+4); // put 4 pixels more instead of 2
        }
        else if (manModel.getState() == ManModel.STATE_MOVING) {
            filterArea.setX(srcX-manModel.getCurrentStepX()-2);
            filterArea.setY(srcY-manModel.getCurrentStepX()-2);
            filterArea.setWidth(2.0*radius+manModel.getCurrentStepX()*2+4);
            filterArea.setHeight(2.0*radius+manModel.getCurrentStepX()*2+4); // put 4 pixels more instead of 2
        }
        else if (manModel.getState() == ManModel.STATE_SLIDING) {
            double yMin = srcY;
            double yMax = srcY + manModel.getDyFall() + 2.0*radius;
            filterArea.setX(0);
            filterArea.setY(yMin);
            filterArea.setWidth(view.getWidth());
            filterArea.setHeight(yMax-yMin);
        }
        else if ((manModel.getState() == ManModel.STATE_FALLING) || (manModel.getState() == ManModel.STATE_JUMPING)) {
            double xMin = srcX<destX?srcX:destX;
            double yMin = srcY<destY?srcY:destY;
            double xMax = srcX>=destX?srcX:destX;
            xMax += 2.0*radius;
            double yMax = srcY>=destY?srcY:destY;
            yMax += 2.0*radius;
            filterArea.setX(xMin);
            filterArea.setY(yMin);
            filterArea.setWidth(xMax-xMin);
            filterArea.setHeight(yMax-yMin);
        }
    }


    // simple case: the hitbox group contains only 1 shape that defines the hitbox
    public Shape getHitBox() {
        return (Shape)(hitBox.getChildren().get(0));
    }

    public void update() {
        if (manModel.isPositionChanged()) {
            body.setTranslateX(Math.floor(manModel.getX()));
            body.setTranslateY(Math.floor(manModel.getY()));
            hitBox.setTranslateX(Math.floor(manModel.getX()));
            hitBox.setTranslateY(Math.floor(manModel.getY()));
        }
        if (manModel.isCollisionOccurred()) {
            outer.setFill(Color.RED);
        }
        if (manModel.isEndCollisionOccurred()) {
            outer.setFill(Color.BLACK);
        }
    }

    public boolean mayCollideWithObstacle(Obstacle o) {
        Shape interShape = Shape.intersect(filterArea, o.getShape());
        Bounds b = interShape.getBoundsInParent();
        if (b.getWidth() != -1) {
            return true;
        }
        return false;
    }

    public CollisionData checkCollision(Obstacle o) {
        double x  = hitBox.getTranslateX();
        double y  = hitBox.getTranslateY();
        Shape interShape = Shape.intersect(getHitBox(), o.getShape());
        Bounds b = interShape.getBoundsInParent();
        if (b.getWidth() != -1) {
            return new CollisionData((int)(b.getMinX()-x), (int)(b.getMinY()-y), (int)(b.getWidth()), (int)(b.getHeight()));
        }
        return new CollisionData(0,0,-1,-1);
    }

    public int getCollisionQuadrants(CollisionData data) {

        int quadrants = 0;

        // if collision occurs on top
        if (data.y <= radius) {
            if (data.x <= radius) {
                quadrants += CollisionData.QUADRANT_TOPLEFT;
            }
            if ((data.x+data.width) >= radius) {
                quadrants += CollisionData.QUADRANT_TOPRIGHT;
            }
        }
        // if collision occurs on bottom
        if (data.y+data.height >= radius) {
            if (data.x <= radius) {
                quadrants += CollisionData.QUADRANT_BOTTOMLEFT;
            }
            if ((data.x+data.width) >= radius) {
                quadrants += CollisionData.QUADRANT_BOTTOMRIGHT;
            }
        }
        return quadrants;
    }

    public int getCollisionWidth(CollisionData data, int baseSide) {
        if (data.width < 0) {
            return -1;
        }
        int width = 0;
        if (baseSide == CollisionData.SIDE_LEFT) {
            width = data.x+data.width;
        }
        else if (baseSide == CollisionData.SIDE_RIGHT) {
            width = (int)(2.0*radius-data.x);
        }
        return width;
    }

    public int getCollisionHeight(CollisionData data, int baseSide) {
        if (data.height < 0) {
            return -1;
        }
        int height = 0;
        if (baseSide == CollisionData.SIDE_TOP) {
            height = data.y+data.height;
        }
        else if (baseSide == CollisionData.SIDE_BOTTOM) {
            height = (int)(2.0*radius-data.y);
        }
        return height;
    }
}
